mod request_builder;
mod types;

use crate::types::AwsRegion;
use request_builder::RequestBuilder;
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt, EnvFilter};

const LOG_FILTER: &str = "DEBUG,rustls=INFO,hyper::proto=INFO";

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    tracing_subscriber::registry()
        .with(EnvFilter::try_from_default_env().unwrap_or(LOG_FILTER.into()))
        .with(tracing_subscriber::fmt::layer().pretty())
        .init();

    let access_id = match std::env::var("AWS_ACCESS_KEY_ID") {
        Ok(x) => x,
        Err(_) => panic!("Missing environment variable 'AWS_ACCESS_KEY_ID'"),
    };

    let access_key = match std::env::var("AWS_SECRET_ACCESS_KEY") {
        Ok(x) => x,
        Err(_) => panic!("Missing environment variable 'AWS_SECRET_ACCESS_KEY'"),
    };

    list_buckets(access_id, access_key).await?;

    Ok(())
}

async fn list_buckets(access_id: String, access_key: String) -> anyhow::Result<()> {
    let req = RequestBuilder::new(
        access_id,
        access_key,
        "https://s3.amazonaws.com/".to_string(),
        AwsRegion::UsEast1,
    );

    let response = req.send().await;

    match response {
        Ok(x) => {
            println!("{}", x.text().await?)
        }
        Err(x) => println!("{:?}", x),
    }

    Ok(())
}
